@@ -1,6 +1,7 @@
 from __future__ import division, print_function, unicode_literals
 
-from contextlib import contextmanager
+from decimal import Decimal
+from fractions import Fraction
 from functools import reduce
 from io import StringIO
 from itertools import chain, count, groupby, permutations, repeat
@@ -619,6 +620,49 @@ class SideEffectTests(TestCase):
         eq_(result, list(range(10)))
         eq_(counter[0], 5)
 
+    def test_before_after(self):
+        f = StringIO()
+        collector = []
+
+        def func(item):
+            print(item, file=f)
+            collector.append(f.getvalue())
+
+        def it():
+            yield u'a'
+            yield u'b'
+            raise Exception('kaboom')
+
+        before = lambda: print('HEADER', file=f)
+        after = f.close
+
+        try:
+            consume(side_effect(func, it(), before=before, after=after))
+        except Exception:
+            pass
+
+        # The iterable should have been written to the file
+        self.assertEqual(collector, [u'HEADER\na\n', u'HEADER\na\nb\n'])
+
+        # The file should be closed even though something bad happened
+        self.assertTrue(f.closed)
+
+    def test_before_fails(self):
+        f = StringIO()
+        func = lambda x: print(x, file=f)
+
+        def before():
+            raise Exception('ouch')
+
+        try:
+            consume(side_effect(func, u'abc', before=before, after=f.close))
+        except Exception:
+            pass
+
+        # The file should be closed even though something bad happened in the
+        # before function
+        self.assertTrue(f.closed)
+
 
 class SlicedTests(TestCase):
     """Tests for ``sliced()``"""
@@ -1109,19 +1153,82 @@ class GroupByTransformTests(TestCase):
         self.assertAllGroupsEqual(actual, expected)
 
 
-class ContextTestCase(TestCase):
+class ArithmeticSequenceTests(TestCase):
     def test_basic(self):
-        before = []
-        after = []
+        for args, expected in [
+            ((4,), [0, 1, 2, 3]),
+            ((4.0,), [0.0, 1.0, 2.0, 3.0]),
+            ((1.0, 4), [1.0, 2.0, 3.0]),
+            ((1, 4.0), [1, 2, 3]),
+            ((1.0, 5), [1.0, 2.0, 3.0, 4.0]),
+            ((0, 20, 5), [0, 5, 10, 15]),
+            ((0, 20, 5.0), [0.0, 5.0, 10.0, 15.0]),
+            ((0, 10, 3), [0, 3, 6, 9]),
+            ((0, 10, 3.0), [0.0, 3.0, 6.0, 9.0]),
+            ((0, -5, -1), [0, -1, -2, -3, -4]),
+            ((0.0, -5, -1), [0.0, -1.0, -2.0, -3.0, -4.0]),
+            ((1, 2, Fraction(1, 2)), [Fraction(1, 1), Fraction(3, 2)]),
+            ((0,), []),
+            ((0.0,), []),
+            ((1, 0), []),
+            ((1.0, 0.0), []),
+            ((Fraction(2, 1),), [Fraction(0, 1), Fraction(1, 1)]),
+            ((Decimal('2.0'),), [Decimal('0.0'), Decimal('1.0')]),
+        ]:
+            actual = list(numeric_range(*args))
+            self.assertEqual(actual, expected)
+            self.assertTrue(
+                all(type(a) == type(e) for a, e in zip(actual, expected))
+            )
 
-        @contextmanager
-        def managed():
-            before.append('open')
-            yield 'running'
-            after.append('close')
+    def test_arg_count(self):
+        self.assertRaises(TypeError, lambda: list(numeric_range()))
+        self.assertRaises(
+            TypeError, lambda: list(numeric_range(0, 1, 2, 3))
+        )
 
-        actual = [(c, x) for c in context(managed()) for x in range(3)]
-        expected = [('running', 0), ('running', 1), ('running', 2)]
+    def test_zero_step(self):
+        self.assertRaises(
+            ValueError, lambda: list(numeric_range(1, 2, 0))
+        )
+
+
+class CountCycleTests(TestCase):
+    def test_basic(self):
+        expected = [
+            (0, 'a'), (0, 'b'), (0, 'c'),
+            (1, 'a'), (1, 'b'), (1, 'c'),
+            (2, 'a'), (2, 'b'), (2, 'c'),
+        ]
+        for actual in [
+            take(9, count_cycle('abc')),  # n=None
+            list(count_cycle('abc', 3)),  # n=3
+        ]:
+            self.assertEqual(actual, expected)
+
+    def test_empty(self):
+        self.assertEqual(list(count_cycle('')), [])
+        self.assertEqual(list(count_cycle('', 2)), [])
+
+    def test_negative(self):
+        self.assertEqual(list(count_cycle('abc', -3)), [])
+
+
+class LocateTests(TestCase):
+    def test_default_pred(self):
+        iterable = [0, 1, 1, 0, 1, 0, 0]
+        actual = list(locate(iterable))
+        expected = [1, 2, 4]
+        self.assertEqual(actual, expected)
+
+    def test_no_matches(self):
+        iterable = [0, 0, 0]
+        actual = list(locate(iterable))
+        expected = []
+
+    def test_custom_pred(self):
+        iterable = ['0', 1, 1, '0', 1, '0', '0']
+        pred = lambda x: x == '0'
+        actual = list(locate(iterable, pred))
+        expected = [0, 3, 5, 6]
         self.assertEqual(actual, expected)
-        self.assertEqual(before, ['open'])
-        self.assertEqual(after, ['close'])
